@@ -722,12 +722,12 @@ class SurfingEntrance(object):
         wechat_bot.send_untreated_new_fund_list(yesterday, untreated_new_funds)
 
     def pull_hedge_fund_nav(self):
-        from .data.manager.manager_fof import FOFDataManager
+        from .data.manager.manager_fof_lite import FOFDataManagerLite
 
         MANAGER_ID = 'py1'
         FOF_ID = 'SLW695'
 
-        fof_dm = FOFDataManager()
+        fof_dm = FOFDataManagerLite()
         fof_dm.pull_hedge_fund_nav(manager_id=MANAGER_ID, fof_id=FOF_ID)
 
 
