@@ -200,7 +200,7 @@ class DerivedDataApi(metaclass=Singleton):
                 return pd.DataFrame([])
 
     def get_index_pair(self):
-        index_val = {'PE':'pe_ttm','PB':'pb_mrq','ROE':'roe'}
+        index_val = {'PE':'pe_ttm_nn','PB':'pb_mrq','ROE':'roe'}
         return index_val
 
     def get_index_val_info(self):
