@@ -346,10 +346,14 @@ class PortfolioDataApi(metaclass=Singleton):
             bk_stats = Calculator.get_stat_result_from_df(df=mv_df.reset_index(), date_column='date', value_column='mv').__dict__
             ar = print_st(bk_stats['annualized_ret'])
             av = print_st(bk_stats['annualized_vol'])
+            r_mdd = print_st(bk_stats['recent_drawdown'])
+            r_mdd_b1 = str(bk_stats['recent_mdd_date1'])
             sharpe = round(bk_stats['sharpe'],2)
             st1 = f'年化收益: {ar} 年化波动: {av} 夏普比率: {sharpe}'
-            st2 = f'最大回撤 {mdd1} {mdd1_d1} : {mdd1_d2}, {mdd1_lens} days'
-            st3 = f'第二回撤 {mdd2} {mdd2_d1} : {mdd2_d2}, {mdd2_lens} days'
+            st2 = f'最近回撤: {r_mdd} 最近回撤开始 {r_mdd_b1}'
+            st3 = f'最大回撤 {mdd1} {mdd1_d1} : {mdd1_d2}, {mdd1_lens} days'
+            st4 = f'第二回撤 {mdd2} {mdd2_d1} : {mdd2_d2}, {mdd2_lens} days'
+            
             mdd_dic_1 = {
                 'd1':str(mdd_details['mdd1_d1']),
                 'd2':str(mdd_details['mdd1_d2']),
@@ -365,11 +369,11 @@ class PortfolioDataApi(metaclass=Singleton):
             data = {
                     'data':df,
                     'mdd_1':mdd_dic_1,
-                    'stats': [st1, st2]
+                    'stats': [st1, st2, st3]
                 }
             if mdd_dic_2['d1'] != 'None':
                 data['mdd_2'] = mdd_dic_2
-                data['stats'].append(st3)
+                data['stats'].append(st4)
             return data
         except Exception as e:
             print(f'Failed to get data <err_msg> {e} from portfolio.get_portfolio_mv')
@@ -421,7 +425,7 @@ class PortfolioDataApi(metaclass=Singleton):
                 data['收益'].index.name = 'datetime'
                 return data
             begin_date, end_date = RawDataApi().get_date_range(time_para, begin_date, end_date)
-            fund_nav = fund_nav.loc[begin_date:end_date]    
+            fund_nav = fund_nav.loc[begin_date:end_date].dropna()    
             fund_nav = CalculatorBase.data_resample_weekly_nav(df=fund_nav, rule=period)
             ret = fund_nav.pct_change(1).iloc[1:]
             ret.columns=['收益']
@@ -705,6 +709,9 @@ class PortfolioDataApi(metaclass=Singleton):
                 dic = ((pd.DataFrame(all_ret_dict).T.fillna(0)+1).prod(axis=1)-1).to_dict()
                 dic = {k:round(v,2) for k,v in dic.items()}
 
+            _df = pd.DataFrame([dic]).T.sort_values(by=0)
+            if _df.shape[0] > 30:
+                dic = pd.concat([_df.iloc[:15], _df.iloc[-15:]]).to_dict()[0]
             return dic
         except Exception as e:
             print(f'Failed to get data <err_msg> {e} from portfolio.get_port_ret_resolve')
@@ -849,7 +856,7 @@ class PortfolioDataApi(metaclass=Singleton):
                                                             start_date=fund_nav.index[0],
                                                             end_date=fund_nav.index[-1],
                                                         )
-            index_price = index_price.pivot_table(index='datetime',columns='index_id',values='close')
+            index_price = index_price.pivot_table(index='datetime',columns='index_id',values='close').ffill()
             fund_info = self.get_all_fund_info(fund_list=fund_nav.columns.tolist())
             fund_desc_dic = fund_info.set_index('fund_id')['desc_name'].to_dict()
             index_info = BasicDataApi().get_index_info(index_list=list(benchmark_id_info.values()))
