@@ -23,6 +23,7 @@ from ..view.basic_models import HedgeFundInfo, FOFInfo, HedgeFundNAV, FOFInvesto
 from ..view.derived_models import FOFNavCalc, FOFPosition, FOFInvestorData, FOFPositionDetail
 from ..wrapper.mysql import BasicDatabaseConnector, DerivedDatabaseConnector
 from ..nav_reader.hedge_fund_nav_reader import HedgeFundNAVReader
+from .manager_hedge_fund import HedgeFundDataManager
 
 
 class FOFDataManagerLite():
@@ -194,14 +195,14 @@ class FOFDataManagerLite():
         # 获取FOF份额变化信息
         fof_scale = DerivedDataApi().get_hedge_fund_investor_pur_redemp(manager_id, [fof_id])
         # 客户认购/申购记录
-        self._fof_scale = fof_scale[fof_scale.event_type.isin([FOFTradeStatus.SUBSCRIBE, FOFTradeStatus.PURCHASE])].set_index('datetime')
+        self._fof_scale = fof_scale[fof_scale.event_type.isin([FOFTradeStatus.SUBSCRIBE, FOFTradeStatus.PURCHASE])].set_index('datetime').sort_index()
         self._start_date = self._fof_scale.index.min()
         # 客户赎回记录
-        self._fof_redemp = fof_scale[fof_scale.event_type.isin([FOFTradeStatus.REDEEM, ])].set_index('datetime')
+        self._fof_redemp = fof_scale[fof_scale.event_type.isin([FOFTradeStatus.REDEEM, ])].set_index('datetime').sort_index()
 
         investor_div_carry = DerivedDataApi().get_hedge_fund_investor_div_carry(manager_id, [fof_id])
         # TODO: 还得处理其他类型的交易
-        self._investor_div_carry = investor_div_carry[investor_div_carry.event_type.isin([FOFTradeStatus.DIVIDEND_VOLUME, ])].set_index('datetime')
+        self._investor_div_carry = investor_div_carry[investor_div_carry.event_type.isin([FOFTradeStatus.DIVIDEND_VOLUME, FOFTradeStatus.DIVIDEND_CASH])].set_index('datetime').sort_index()
 
         # trading_day_list = BasicDataApi().get_trading_day_list(start_date=self._start_date, end_date=datetime.datetime.now().date())
         # 将昨天作为end_date
@@ -211,7 +212,7 @@ class FOFDataManagerLite():
         self._date_list: np.ndarray = pd.date_range(self._start_date, self._end_date).date
 
         # 获取fof持仓
-        self._asset_allocation: Optional[pd.DataFrame] = FOFDataManagerLite.get_fof_asset_allocation(manager_id, [fof_id])
+        self._asset_allocation: Optional[pd.DataFrame] = FOFDataManagerLite.get_fof_asset_allocation(manager_id, [fof_id]).sort_values(by='datetime')
         assert self._asset_allocation is not None, f'get fof pos for {manager_id}/{fof_id} failed'
 
         positions = self._asset_allocation.loc[self._asset_allocation.event_type.isin([FOFTradeStatus.PURCHASE, FOFTradeStatus.SUBSCRIBE]), :]
@@ -541,15 +542,26 @@ class FOFDataManagerLite():
         if debug_mode:
             total_info = pd.concat([shares_list, other_assets_list, fof_nav_list, today_fund_mv_list, today_hedge_mv_list, net_assets_list, net_assets_fixed_list, deposit_interest_list, positions_list], axis=1).sort_index()
             print(total_info)
-        self._fof_nav = pd.concat([fof_nav_list, net_assets_fixed_list, shares_list.cumsum(), deposit_interest_list, management_fee_list, custodian_fee_list, administrative_fee_list], axis=1).sort_index().rename_axis('datetime').reset_index()
+        self._fof_nav = pd.concat([fof_nav_list, net_assets_fixed_list, shares_list.cumsum(), deposit_interest_list, management_fee_list, custodian_fee_list, administrative_fee_list], axis=1).sort_index().rename_axis('datetime')
         self._fof_nav['share'] = self._fof_nav.share.ffill()
-        self._fof_nav = self._fof_nav[self._fof_nav.datetime >= self._ESTABLISHED_DATE]
+        self._fof_nav = self._fof_nav[self._fof_nav.index >= self._ESTABLISHED_DATE]
         self._fof_nav['fof_id'] = fof_id
         self._fof_nav['manager_id'] = manager_id
-        # TODO: 暂时先直接用 nav 给 acc_net_value 和 adjusted_nav 赋值
-        self._fof_nav['acc_net_value'] = self._fof_nav.nav
-        self._fof_nav['adjusted_nav'] = self._fof_nav.nav
+
+        dividend_record = self._investor_div_carry.loc[~self._investor_div_carry.index.duplicated(), :].drop_duplicates(subset=['fof_id', 'manager_id'])
+        if dividend_record.empty:
+            self._fof_nav['acc_net_value'] = self._fof_nav.nav
+            self._fof_nav['adjusted_nav'] = self._fof_nav.nav
+            self._fof_nav['ta_factor'] = 1
+        else:
+            dividend_cum = (dividend_record.acc_unit_value - dividend_record.net_asset_value)
+            dividend_sep = (dividend_cum - dividend_cum.shift(1, fill_value=0)).dropna().reindex(self._fof_nav.index, method='ffill')
+            self._fof_nav['acc_net_value'] = self._fof_nav.nav.add(dividend_sep, fill_value=0)
+            adjusted_nav_and_ta_factor = HedgeFundDataManager.calc_whole_adjusted_net_value(self._fof_nav[['nav', 'acc_net_value']].rename(columns={'nav': 'net_asset_value', 'acc_net_value': 'acc_unit_value'}))
+            self._fof_nav['adjusted_nav'], self._fof_nav['ta_factor'] = adjusted_nav_and_ta_factor['adj_nav'], adjusted_nav_and_ta_factor['adj_factor']
+
         self._fof_nav['ret'] = self._fof_nav.adjusted_nav / self._fof_nav.adjusted_nav.array[0] - 1
+        self._fof_nav = self._fof_nav.reset_index()
 
         self._fof_position = pd.concat([positions_list, other_assets_list.rename('other_assets')], axis=1).rename_axis('datetime').reset_index()
         self._fof_position = self._fof_position[self._fof_position.position.notna()]
@@ -668,7 +680,6 @@ class FOFDataManagerLite():
             print('[dump_fof_nav_and_pos_to_db] no position, should calc it first')
 
         if self._fof_position_details is not None and not self._fof_position_details.empty:
-            print(self._fof_position_details)
             now_df = DerivedDataApi().get_fof_position_detail(manager_id, [fof_id])
             if now_df is not None:
                 now_df = now_df.drop(columns=['update_time', 'create_time', 'is_deleted']).astype(self._fof_position_details.dtypes.to_dict())
@@ -759,7 +770,7 @@ class FOFDataManagerLite():
         #                 'sharpe': round(res_status.sharpe, 6),
         #                 'annualized_ret': f'{round(res_status.annualized_ret * 100, 2)}%',
         #                 'mdd': f'{round(res_status.mdd * 100, 2)}%',
-        #                 'annualized_vol': round(res_status.annualized_vol, 6),
+        #                 'annualized_vol': f'{round(res_status.annualized_vol * 100, 2)}%',
         #             }
         #             self._wechat_bot.send_fof_info(fof_id, fof_nav.index.array[-1], fof_data)
         #         except Exception as e:
@@ -778,9 +789,9 @@ class FOFDataManagerLite():
         # 获取FOF份额变化信息
         fof_scale = DerivedDataApi().get_hedge_fund_investor_pur_redemp(manager_id, [fof_id])
         # 客户认购/申购记录
-        self._fof_scale = fof_scale[fof_scale.event_type.isin([FOFTradeStatus.SUBSCRIBE, FOFTradeStatus.PURCHASE])].set_index('datetime')
+        self._fof_scale = fof_scale[fof_scale.event_type.isin([FOFTradeStatus.SUBSCRIBE, FOFTradeStatus.PURCHASE])].set_index('datetime').sort_index()
         # 客户赎回记录
-        self._fof_redemp = fof_scale[fof_scale.event_type.isin([FOFTradeStatus.REDEEM, ])].set_index('datetime')
+        self._fof_redemp = fof_scale[fof_scale.event_type.isin([FOFTradeStatus.REDEEM, ])].set_index('datetime').sort_index()
         # fof_nav数据优先级更大
         fof_nav = FOFDataManagerLite.get_fof_nav(manager_id=manager_id, fof_id=fof_id)
         fof_nav_public = FOFDataManagerLite.get_fof_nav_public(manager_id=manager_id, fof_id=fof_id)
@@ -830,7 +841,7 @@ class FOFDataManagerLite():
                     'sharpe': round(res_status.sharpe, 6),
                     'annualized_ret': f'{round(res_status.annualized_ret * 100, 2)}%',
                     'mdd': f'{round(res_status.mdd * 100, 2)}%',
-                    'annualized_vol': round(res_status.annualized_vol, 6),
+                    'annualized_vol': f'{round(res_status.annualized_vol * 100, 2)}%',
                 }
 
                 latest_nav_date = fund_nav.loc[:, 'datetime'].iat[-1]
@@ -885,7 +896,7 @@ class FOFDataManagerLite():
                         'sharpe': round(res_status.sharpe, 6),
                         'annualized_ret': the_xirr,
                         'mdd': f'{round(res_status.mdd * 100, 2)}%',
-                        'annualized_vol': round(res_status.annualized_vol, 6),
+                        'annualized_vol': f'{round(res_status.annualized_vol * 100, 2)}%',
                     }
                     self._wechat_bot.send_hedge_fund_info(fof_id, fund_id, hedge_info.at[fund_id, 'brief_name'], latest_nav_date, fund_data, holding_data, ret_data_with_last_date, ret_date_with_last_pub)
             except Exception as e:
@@ -1125,18 +1136,21 @@ class FOFDataManagerLite():
                 the_amount = row.purchase_amount
                 if row.event_type == FOFTradeStatus.PURCHASE:
                     the_amount += row.raising_interest
-                investor_mvs[row.investor_id].append({'trade_ids': [row.id], 'start_date': actual_start_date, 'amount': the_amount, 'left_amount': the_amount, 'share': row.share_changed, 'left_share': row.share_changed, 'init_mv': init_mv, 'water_line': row.acc_unit_value})
+                investor_mvs[row.investor_id].append({'trade_ids': [row.id], 'start_date': actual_start_date, 'amount': the_amount, 'left_amount': the_amount, 'share': row.share_changed, 'left_share': row.share_changed, 'init_mv': init_mv,
+                                                      'water_line': row.acc_unit_value, 'redemp_confirmed_amount': 0, 'dividend_cash': 0})
             elif row.event_type in (FOFTradeStatus.REDEEM, ):
-                share_redemp = row.share_changed
+                share_redemp = abs(row.share_changed)
                 for one in investor_mvs[row.investor_id]:
-                    if one['left_share'] >= abs(share_redemp):
-                        one['left_share'] -= abs(share_redemp)
+                    if one['left_share'] >= share_redemp:
+                        one['left_share'] -= share_redemp
                         one['trade_ids'].append(row.id)
+                        # FIXME: 在这里记一下赎回金额
+                        one['redemp_confirmed_amount'] += row.redemp_confirmed_amount
                         break
-                    else:
-                        one['left_share'] = 0
-                        one['left_amount'] = 0
-                        one['trade_ids'].append(row.id)
+                    share_redemp -= one['left_share']
+                    one['left_share'] = 0
+                    one['left_amount'] = 0
+                    one['trade_ids'].append(row.id)
 
         for row in investor_div_carry.itertuples(index=False):
             if investor_id_list and row.investor_id not in investor_id_list:
@@ -1144,14 +1158,20 @@ class FOFDataManagerLite():
             # TODO: 需要处理其他几种类型的交易
             if row.event_type in (FOFTradeStatus.DIVIDEND_VOLUME, ):
                 assert row.investor_id in investor_mvs, '!!!'
-                share_dividend = row.share_changed
                 is_added = False
                 for one in investor_mvs[row.investor_id]:
                     if not is_added:
-                        one['left_share'] += share_dividend
+                        one['left_share'] += row.share_changed
+                        one['left_amount'] += row.reinvest_amount
                         is_added = True
                     # 水位线都改成当前的这个
                     one['water_line'] = row.acc_unit_value
+            elif row.event_type in (FOFTradeStatus.DIVIDEND_CASH, ):
+                assert row.investor_id in investor_mvs, '!!!'
+                for one in investor_mvs[row.investor_id]:
+                    # FIXME: 先找个地方存一下现金分红
+                    one['dividend_cash'] += row.cash_dividend
+                    break
 
         investor_returns = {}
         latest_nav = fof_nav.nav.array[-1]
@@ -1166,7 +1186,7 @@ class FOFDataManagerLite():
             avg_v_nav = latest_mv / total_share
             amount_sum = datas.amount.sum()
             if not math.isclose(amount_sum, 0):
-                total_ret = latest_mv - amount_sum
+                total_ret = latest_mv + datas.redemp_confirmed_amount.sum() + datas.dividend_cash.sum() - amount_sum
                 total_rr = total_ret / amount_sum
             else:
                 total_ret = np.nan
@@ -1176,8 +1196,8 @@ class FOFDataManagerLite():
             else:
                 avg_nav_cost = np.nan
             investor_returns[investor_id] = {
-                'datetime': fof_nav.index.array[-1], 'manager_id': manager_id, 'fof_id': fof_id, 'v_nav': avg_v_nav, 'cost_nav': avg_nav_cost, 'total_ret': total_ret, 'total_rr': total_rr, 'amount': amount_sum, 'shares': total_share, 'latest_mv': latest_mv,
-                'details': datas[['trade_ids', 'amount', 'left_share', 'water_line']].rename(columns={'left_share': 'share'}).join(mv.rename('mv')).to_json(orient='records')
+                'datetime': fof_nav.index.array[-1], 'manager_id': manager_id, 'fof_id': fof_id, 'v_nav': avg_v_nav, 'cost_nav': avg_nav_cost, 'total_ret': total_ret, 'total_rr': total_rr, 'amount': amount_sum, 'shares': total_share,
+                'latest_mv': latest_mv, 'details': datas[['trade_ids', 'amount', 'left_share', 'water_line']].rename(columns={'left_share': 'share'}).join(mv.rename('mv')).to_json(orient='records')
             }
         return pd.DataFrame.from_dict(investor_returns, orient='index').rename_axis(index='investor_id')
 
