@@ -59,6 +59,10 @@ class SeriesStatUnit:
     cvar: float                         # 条件风险价值 CVaR
     raise_month_num: int                # 上涨月份数
     drop_month_num: int                 # 下跌月份数
+    mdd_recover: float                  # 最大回撤恢复
+    mdd_recover_date1: datetime.date    # 最大回撤恢复开始时间
+    mdd_recover_date2: datetime.date    # 最大回测恢复结束时间
+    mdd_recover_lens: int               # 最大回测哈恢复天数
 
 
 @dataclasses.dataclass
@@ -188,6 +192,10 @@ class Calculator:
             recent_year_ret=res['recent_year_ret'],
             raise_month_num=res['raise_month_num'],
             drop_month_num=res['drop_month_num'],
+            mdd_recover=res['mdd_recover'],
+            mdd_recover_date1=res['mdd_recover_date1'],
+            mdd_recover_date2=res['mdd_recover_date2'],
+            mdd_recover_lens=res['mdd_recover_lens'],
         )
 
     @staticmethod
@@ -280,6 +288,10 @@ class Calculator:
             kurtosis=res['kurtosis'],
             cumu_ret=res['cumu_ret'],
             total_relative_ret=res['total_relative_ret'],
+            mdd_recover=res['mdd_recover'],
+            mdd_recover_date1=res['mdd_recover_date1'],
+            mdd_recover_date2=res['mdd_recover_date2'],
+            mdd_recover_lens=res['mdd_recover_lens'],
         )
 
     @staticmethod
