@@ -449,6 +449,7 @@ class CalculatorBase:
             sr_ret_monthly = CalculatorBase.data_resample_monthly_ret(df=sr_ret, min_count=1).dropna()
             raise_month_num = sr_ret_monthly[sr_ret_monthly>0].shape[0]
             drop_month_num = sr_ret_monthly[sr_ret_monthly<0].shape[0]
+            mdd_recover, mdd_recover_date1, mdd_recover_date2, mdd_recover_lens = CalculatorBase.mdd_recover_analysis(values=values, dates=dates)
 
             if benchmark_values is not None:
                 assert len(dates) == len(benchmark_values), 'date and bench_values has different numbers'
@@ -616,6 +617,10 @@ class CalculatorBase:
                     'excess_mdd_date2':excess_mdd_date2,
                     'excess_mdd_lens':excess_mdd_lens,
                     'total_relative_ret':total_relative_ret,
+                    'mdd_recover':mdd_recover,
+                    'mdd_recover_date1':mdd_recover_date1,
+                    'mdd_recover_date2':mdd_recover_date2,
+                    'mdd_recover_lens':mdd_recover_lens,
                 }
             else:
                 return {
@@ -659,6 +664,10 @@ class CalculatorBase:
                     'ervar':ervar,
                     'skew':skew_value,
                     'kurtosis':kurtosis_value,
+                    'mdd_recover':mdd_recover,
+                    'mdd_recover_date1':mdd_recover_date1,
+                    'mdd_recover_date2':mdd_recover_date2,
+                    'mdd_recover_lens':mdd_recover_lens,
                 }
         else:
             return {
@@ -734,8 +743,40 @@ class CalculatorBase:
                     'excess_mdd_date2':dates[0] if len(dates) > 0 else None,
                     'excess_mdd_lens':0,
                     'total_relative_ret':0,
+                    'mdd_recover':0,
+                    'mdd_recover_date1':dates[0] if len(dates) > 0 else None,
+                    'mdd_recover_date2':dates[0] if len(dates) > 0 else None,
+                    'mdd_recover_lens':0,
                 }
 
+
+    @staticmethod    
+    def mdd_recover_analysis(values,dates):
+        sr = pd.Series(values, index=dates).sort_index()
+        if sr.empty:
+            mdd = 0
+            mdd_date1 = None
+            mdd_date2 = None
+            mdd_lens = 0
+            return mdd, mdd_date1, mdd_date2, mdd_lens
+        mdd_part =  sr[:] / sr[:].rolling(window=sr.shape[0], min_periods=1).max()
+        mdd = 1 - mdd_part.min()
+        if mdd == 0:
+            mdd_date1 = None
+            mdd_date2 = None
+            mdd_lens = 0
+        else:
+            mdd_date = mdd_part.idxmin()
+            mdd_date1 = sr[:mdd_date].idxmax()
+            sr_tmp = sr[mdd_date1:]
+            recover_sr = sr_tmp[sr_tmp> sr[mdd_date1]]
+            if recover_sr.empty:
+                mdd_date2 = sr_tmp.index[-1]
+            else: 
+                mdd_date2 = sr_tmp[sr_tmp> sr[mdd_date1]].index[0]
+            mdd_lens = sr.loc[mdd_date1:mdd_date2].shape[0]
+        return mdd, mdd_date1, mdd_date2, mdd_lens
+
     @staticmethod
     def simulated_mv_calc(df,fee_dic,weight_dic):
         '''
